@@ -18,6 +18,7 @@ from typing import Callable, Generator, List, Union
 import requests
 from requests.exceptions import Timeout
 
+from . import macro
 from .client import IOTBOT
 from .config import config
 from .logger import logger
@@ -84,7 +85,7 @@ class _SendThread(threading.Thread):
             self.start()
 
 
-class Action:  # pylint:disable=too-many-instance-attributes
+class Action:  # pylint:disable=R0904
     """
     :param qq_or_bot: qq号或者机器人实例(`IOTBOT`)
     :param queue: 是否开启队列，开启后任务将按顺序发送并延时指定时间，此参数与`queue_delay`对应
@@ -138,7 +139,6 @@ class Action:  # pylint:disable=too-many-instance-attributes
             "sendMsgType": "TextMsg",
             "content": content,
             "groupid": 0,
-            "atUser": 0,
             "replayInfo": None,
         }
         return self.baseSender('POST', 'SendMsg', data, timeout, **kwargs)
@@ -159,7 +159,6 @@ class Action:  # pylint:disable=too-many-instance-attributes
             "sendMsgType": "VoiceMsg",
             "content": "",
             "groupid": 0,
-            "atUser": 0,
             "voiceUrl": voiceUrl,
             "voiceBase64Buf": voiceBase64Buf,
         }
@@ -171,7 +170,7 @@ class Action:  # pylint:disable=too-many-instance-attributes
         content='',
         picUrl='',
         picBase64Buf='',
-        fileMd5='',
+        fileMd5: List[str] = None,
         flashPic=False,
         timeout=5,
         **kwargs,
@@ -183,7 +182,6 @@ class Action:  # pylint:disable=too-many-instance-attributes
             "sendMsgType": "PicMsg",
             "content": content,
             "groupid": 0,
-            "atUser": 0,
             "picUrl": picUrl,
             "picBase64Buf": picBase64Buf,
             "fileMd5": fileMd5,
@@ -192,16 +190,22 @@ class Action:  # pylint:disable=too-many-instance-attributes
         return self.baseSender('POST', 'SendMsg', data, timeout, **kwargs)
 
     def send_group_text_msg(
-        self, toUser: int, content='', atUser=0, timeout=5, **kwargs
+        self,
+        toUser: int,
+        content='',
+        atUser: Union[int, List[int]] = 0,
+        timeout=5,
+        **kwargs,
     ) -> dict:
         """发送群文字消息"""
+        if atUser != 0:
+            content = macro.atUser(atUser) + content
         data = {
             "toUser": toUser,
             "sendToType": 2,
             "sendMsgType": "TextMsg",
             "content": content,
             "groupid": 0,
-            "atUser": atUser,
         }
         return self.baseSender('POST', 'SendMsg', data, timeout, **kwargs)
 
@@ -215,7 +219,6 @@ class Action:  # pylint:disable=too-many-instance-attributes
             "sendMsgType": "VoiceMsg",
             "content": '',
             "groupid": 0,
-            "atUser": 0,
             "voiceUrl": voiceUrl,
             "voiceBase64Buf": voiceBase64Buf,
         }
@@ -226,33 +229,29 @@ class Action:  # pylint:disable=too-many-instance-attributes
         toUser: int,
         picUrl='',
         flashPic=False,
-        atUser=0,
+        atUser: Union[int, List[int]] = 0,
         content='',
         picBase64Buf='',
-        fileMd5='',
+        fileMd5: List[str] = None,  # 多图
         timeout=5,
         **kwargs,
     ) -> dict:
         """发送群图片
         Tips:
             [秀图id] 各id对应效果
-            40000   秀图
-            40001   幻影
-            40002   抖动
-            40003   生日
-            40004   爱你
-            40005   征友
-            40006   无(只显示大图无特效)
+            40000 秀图  40001 幻影  40002 抖动 40003 生日
+            40004 爱你  40005 征友  40006 无(只显示大图无特效)
 
             [PICFLAG] 改变图文消息顺序
         """
+        if atUser != 0:
+            content = macro.atUser(atUser) + content
         data = {
             "toUser": toUser,
             "sendToType": 2,
             "sendMsgType": "PicMsg",
             "content": content,
             "groupid": 0,
-            "atUser": atUser,
             "picUrl": picUrl,
             "picBase64Buf": picBase64Buf,
             "fileMd5": fileMd5,
@@ -270,7 +269,6 @@ class Action:  # pylint:disable=too-many-instance-attributes
             "sendMsgType": "TextMsg",
             "content": content,
             "groupid": groupid,
-            "atUser": 0,
         }
         return self.baseSender('POST', 'SendMsg', data, timeout, **kwargs)
 
@@ -284,7 +282,6 @@ class Action:  # pylint:disable=too-many-instance-attributes
             "sendMsgType": "VoiceMsg",
             "content": "",
             "groupid": groupid,
-            "atUser": 0,
             "voiceUrl": voiceUrl,
             "voiceBase64Buf": voiceBase64Buf,
         }
@@ -297,7 +294,7 @@ class Action:  # pylint:disable=too-many-instance-attributes
         picUrl='',
         picBase64Buf='',
         content='',
-        fileMd5='',
+        fileMd5: List[str] = None,
         timeout=10,
         **kwargs,
     ) -> dict:
@@ -308,16 +305,13 @@ class Action:  # pylint:disable=too-many-instance-attributes
             "sendMsgType": "PicMsg",
             "content": content,
             "groupid": groupid,
-            "atUser": 0,
             "picUrl": picUrl,
             "picBase64Buf": picBase64Buf,
             "fileMd5": fileMd5,
         }
         return self.baseSender('POST', 'SendMsg', data, timeout, **kwargs)
 
-    def send_group_json_msg(
-        self, toUser: int, content='', atUser=0, timeout=5, **kwargs
-    ) -> dict:
+    def send_group_json_msg(self, toUser: int, content='', timeout=5, **kwargs) -> dict:
         """发送群Json类型信息
         :param content: 可以为json文本，或者字典类型
         """
@@ -329,13 +323,10 @@ class Action:  # pylint:disable=too-many-instance-attributes
             "sendMsgType": "JsonMsg",
             "content": content,
             "groupid": 0,
-            "atUser": atUser,
         }
         return self.baseSender('POST', 'SendMsg', data, timeout, **kwargs)
 
-    def send_group_xml_msg(
-        self, toUser: int, content='', atUser=0, timeout=5, **kwargs
-    ) -> dict:
+    def send_group_xml_msg(self, toUser: int, content='', timeout=5, **kwargs) -> dict:
         """发送群Xml类型信息"""
         data = {
             "toUser": toUser,
@@ -343,7 +334,6 @@ class Action:  # pylint:disable=too-many-instance-attributes
             "sendMsgType": "XmlMsg",
             "content": content,
             "groupid": 0,
-            "atUser": atUser,
         }
         return self.baseSender('POST', 'SendMsg', data, timeout, **kwargs)
 
@@ -643,7 +633,6 @@ class Action:  # pylint:disable=too-many-instance-attributes
             "sendMsgType": "ForwordMsg",
             "content": "",
             "groupid": 0,
-            "atUser": 0,
             "forwordBuf": forwordBuf,
             "forwordField": 19,
         }
@@ -659,7 +648,6 @@ class Action:  # pylint:disable=too-many-instance-attributes
             "sendMsgType": "ForwordMsg",
             "content": "",
             "groupid": 0,
-            "atUser": 0,
             "forwordBuf": forwordBuf,
             "forwordField": 19,
         }
@@ -680,6 +668,32 @@ class Action:  # pylint:disable=too-many-instance-attributes
                 logger.error('base64获取失败')
         return ''
 
+    def get_schedules(self, **kwargs) -> dict:
+        """获取定时任务总数
+        response {"Crons": "任务总数0\n", "Ret": 0}
+        """
+        return self.baseSender('GET', 'GetCrons', **kwargs)
+
+    def add_schedules(self, Sepc: str, FileName: str, FuncName: str, **kwargs) -> dict:
+        """添加定时任务
+        :param sepc: cron表达式
+        :param FileName: 执行的lua文件名
+        :param FuncName: 指定在该lua文件下的方法
+        """
+        data = {
+            "QQ": str(self.qq),  # 执行任务的机器人
+            "Sepc": Sepc,  # cron表达式 每5秒执行一次
+            "FileName": FileName,  # 执行的lua文件名
+            "FuncName": FuncName,  # 执行的lua文件名下的TaskTwo方法名
+        }
+        return self.baseSender('POST', 'AddCrons', data, **kwargs)
+
+    def del_schedules(self, TaskID: int, **kwargs) -> dict:
+        """删除定时任务
+        :param TaskID: 任务ID
+        """
+        return self.baseSender('POST', 'DelCrons', {'TaskID': TaskID}, **kwargs)
+
     def baseSender(
         self,
         method: str,
